import type { ThemedToken } from '@shikijs/types'
import type { SyntaxHighlighterProps, TokensResult } from '../types/shiki'
import { useEffect, useState } from 'react'
import { createHighlighter } from '../syntax/highlighter'

export function useSyntaxHighlighter({ text, language, languageId, theme }: SyntaxHighlighterProps) {
  const [tokens, setTokens] = useState<ThemedToken[][]>([])
  const [error, setError] = useState<string>()
  const [isLoading, setIsLoading] = useState(true)

  useEffect(() => {
    let mounted = true
    setIsLoading(true)

    async function highlight() {
      if (!text) {
        setIsLoading(false)
        setTokens([])
        return
      }

      try {
        const highlighter = await createHighlighter({
          langs: [language],
          themes: theme ? [theme] : [],
        })

        const result = await highlighter.codeToTokens(text, {
          lang: languageId,
          theme,
          includeExplanation: true,
        }) as TokensResult

        // Log scopes from explanations if they're being generated correctly
        result.tokens.forEach((lineTokens, _lineIndex) => {
          lineTokens.forEach((token) => {
            const explanations = token.explanation?.map(explain => explain.scopes) || []
            console.log('Token scopes:', {
              content: token.content,
              scopes: explanations,
              color: token.color,
            })
          })
        })

        if (mounted && result.tokens) {
          setTokens(result.tokens)
          setError(undefined)
        }
      }
      catch (err) {
        console.error('Highlighting error:', err)
        if (mounted) {
          setError(err instanceof Error ? err.message : 'Syntax highlighting failed')
          setTokens([])
        }
      }
      finally {
        if (mounted) {
          setIsLoading(false)
        }
      }
    }

    highlight()
    return () => {
      mounted = false
    }
  }, [text, language, languageId, theme])

  return { tokens, error, isLoading }
}
